@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,6 +23,8 @@ module variable references.
 
 """
 
+from nuitka.__past__ import iterItems
+from nuitka.nodes.shapes.StandardShapes import ShapeUnknown
 from nuitka.utils import InstanceCounters, Utils
 
 complete = False
@@ -66,6 +68,9 @@ class Variable(object):
     def getOwner(self):
         return self.owner
 
+    def getEntryPoint(self):
+        return self.owner.getEntryPoint()
+
     def getCodeName(self):
         var_name = self.variable_name
         var_name = var_name.replace('.', '$')
@@ -119,15 +124,19 @@ class Variable(object):
         if not self.users:
             return False
 
+        owner = self.owner.getEntryPoint()
+
         for user in self.users:
-            while user is not self.owner and \
+            user = user.getEntryPoint()
+
+            while user is not owner and \
                   (
                    (user.isExpressionFunctionBody() and not user.needsCreation()) or \
                    user.isExpressionClassBody()
                   ):
                 user = user.getParentVariableProvider()
 
-            if user is not self.owner:
+            if user is not owner:
                 return True
 
         return False
@@ -137,7 +146,6 @@ class Variable(object):
 
     def removeTrace(self, variable_trace):
         # Make it unusable, and break GC cycles while at it.
-        variable_trace.variable = None
         variable_trace.previous = None
 
         self.traces.remove(variable_trace)
@@ -187,6 +195,28 @@ class Variable(object):
 
         return None
 
+    def getTypeShapes(self):
+        result = set()
+
+        for trace in self.traces:
+            if trace.isAssignTrace():
+                result.add(
+                    trace.getAssignNode().getAssignSource().getTypeShape()
+                )
+            elif trace.isUnknownTrace():
+                result.add(ShapeUnknown)
+            elif trace.isUninitTrace():
+                if trace.hasDefiniteUsages() or trace.hasPotentialUsages():
+                    result.add(ShapeUnknown)
+            elif trace.isInitTrace():
+                result.add(ShapeUnknown)
+            elif trace.isMergeTrace():
+                pass
+            else:
+                assert False, trace
+
+        return result
+
 
 class LocalVariable(Variable):
     __slots__ = ()
@@ -227,7 +257,7 @@ class ParameterVariable(LocalVariable):
 
 
 class ModuleVariable(Variable):
-    __slots__ = "module",
+    __slots__ = ("module",)
 
     def __init__(self, module, variable_name):
         assert type(variable_name) is str, repr(variable_name)
@@ -270,7 +300,7 @@ class TempVariable(Variable):
     def __repr__(self):
         return "<TempVariable '%s' of '%s'>" % (
             self.getName(),
-            self.getOwner()
+            self.getOwner().getName()
         )
 
     def getDescription(self):
@@ -280,22 +310,18 @@ class TempVariable(Variable):
         return True
 
 
-def updateFromCollection(old_collection, new_collection):
+def updateVariablesFromCollection(old_collection, new_collection):
     # After removing/adding traces, we need to pre-compute the users state
     # information.
     touched_variables = set()
 
     if old_collection is not None:
-        for variable_trace in old_collection.getVariableTracesAll().values():
-            variable = variable_trace.getVariable()
-
+        for (variable, _version), variable_trace in iterItems(old_collection.getVariableTracesAll()):
             variable.removeTrace(variable_trace)
             touched_variables.add(variable)
 
     if new_collection is not None:
-        for variable_trace in new_collection.getVariableTracesAll().values():
-            variable = variable_trace.getVariable()
-
+        for (variable, _version), variable_trace in iterItems(new_collection.getVariableTracesAll()):
             variable.addTrace(variable_trace)
             touched_variables.add(variable)
 
