@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,9 +21,10 @@ This is code generation for built-in references, and some built-ins like range,
 bin, etc.
 """
 from nuitka import Builtins
+from nuitka.PythonVersions import python_version
 
+from .CodeHelpers import generateChildExpressionsCode
 from .ErrorCodes import getAssertionCode, getErrorExitCode, getReleaseCodes
-from .Helpers import generateChildExpressionsCode
 from .PythonAPICodes import generateCAPIObjectCode, generateCAPIObjectCode0
 
 
@@ -109,14 +110,25 @@ def generateBuiltinType3Code(to_name, expression, emit, context):
 
 
 def generateBuiltinOpenCode(to_name, expression, emit, context):
+    arg_desc = (
+        ("open_filename", expression.getFilename()),
+        ("open_mode", expression.getMode()),
+        ("open_buffering", expression.getBuffering())
+    )
+
+    if python_version >= 300:
+        arg_desc += (
+            ("open_encoding", expression.getEncoding()),
+            ("open_errors", expression.getErrors()),
+            ("open_newline", expression.getNewline()),
+            ("open_closefd", expression.getCloseFd()),
+            ("open_opener", expression.getOpener())
+        )
+
     generateCAPIObjectCode(
         to_name    = to_name,
         capi       = "BUILTIN_OPEN",
-        arg_desc   = (
-            ("open_filename", expression.getFilename()),
-            ("open_mode", expression.getMode()),
-            ("open_buffering", expression.getBuffering()),
-        ),
+        arg_desc   = arg_desc,
         may_raise  = expression.mayRaiseException(BaseException),
         none_null  = True,
         source_ref = expression.getCompatibleSourceReference(),
@@ -333,10 +345,10 @@ def generateBuiltinHexCode(to_name, expression, emit, context):
     )
 
 
-def generateBuiltinBytearrayCode(to_name, expression, emit, context):
+def generateBuiltinBytearray1Code(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name    = to_name,
-        capi       = "BUILTIN_BYTEARRAY",
+        capi       = "BUILTIN_BYTEARRAY1",
         arg_desc   = (
             ("bytearray_arg", expression.getValue()),
         ),
@@ -345,3 +357,48 @@ def generateBuiltinBytearrayCode(to_name, expression, emit, context):
         emit       = emit,
         context    = context
     )
+
+
+def generateBuiltinBytearray3Code(to_name, expression, emit, context):
+    generateCAPIObjectCode(
+        to_name    = to_name,
+        capi       = "BUILTIN_BYTEARRAY3",
+        arg_desc   = (
+            ("bytearray_string", expression.getStringArg()),
+            ("bytearray_encoding", expression.getEncoding()),
+            ("bytearray_errors", expression.getErrors()),
+        ),
+        may_raise  = expression.mayRaiseException(BaseException),
+        source_ref = expression.getCompatibleSourceReference(),
+        none_null  = True,
+        emit       = emit,
+        context    = context
+    )
+
+
+def generateBuiltinStaticmethodCode(to_name, expression, emit, context):
+    generateCAPIObjectCode(
+        to_name    = to_name,
+        capi       = "BUILTIN_STATICMETHOD",
+        arg_desc   = (
+            ("staticmethod_arg", expression.getValue()),
+        ),
+        may_raise  = expression.mayRaiseException(BaseException),
+        source_ref = expression.getCompatibleSourceReference(),
+        emit       = emit,
+        context    = context
+    )
+
+
+def generateBuiltinClassmethodCode(to_name, expression, emit, context):
+    generateCAPIObjectCode(
+        to_name    = to_name,
+        capi       = "BUILTIN_CLASSMETHOD",
+        arg_desc   = (
+            ("classmethod_arg", expression.getValue()),
+        ),
+        may_raise  = expression.mayRaiseException(BaseException),
+        source_ref = expression.getCompatibleSourceReference(),
+        emit       = emit,
+        context    = context
+    )
