@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,8 +23,8 @@ of the metaclass remains as specific.
 
 from nuitka.PythonVersions import python_version
 
+from .CodeHelpers import generateChildExpressionsCode
 from .ErrorCodes import getErrorExitCode, getReleaseCode, getReleaseCodes
-from .Helpers import generateChildExpressionsCode
 from .PythonAPICodes import generateCAPIObjectCode0
 
 
