@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -63,8 +63,10 @@ def getFrameVariableTypeDescriptionCode(context):
     type_description = context.getFrameVariableTypeDescription()
 
     if type_description:
-        context.markAsNeedsFrameVariableDescription()
-        return 'type_description = "%s";' % type_description
+        return '%s = "%s";' % (
+            context.getFrameVariableTypeDescriptionName(),
+            type_description,
+        )
     else:
         return ""
 
@@ -290,7 +292,7 @@ def getReleaseCodes(release_names, emit, context):
 def getMustNotGetHereCode(reason, context, emit):
     getCommentCode(reason, emit)
 
-    provider = context.getOwner()
+    provider = context.getEntryPoint()
 
     emit(
         "NUITKA_CANNOT_GET_HERE( %(function_identifier)s );" % {
