@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,13 +20,13 @@
 That is import as expression, and star import.
 """
 
+from .CodeHelpers import generateChildExpressionsCode, generateExpressionCode
 from .ErrorCodes import (
     getErrorExitBoolCode,
     getErrorExitCode,
     getReleaseCode,
     getReleaseCodes
 )
-from .Helpers import generateChildExpressionsCode, generateExpressionCode
 from .LineNumberCodes import emitLineNumberUpdateCode
 from .ModuleCodes import getModuleAccessCode
 
@@ -125,18 +125,31 @@ def getBuiltinImportCode(to_name, module_name, globals_name, locals_name,
 
 
 def generateImportModuleHardCode(to_name, expression, emit, context):
-    getImportModuleHardCode(
-        to_name     = to_name,
-        module_name = expression.getModuleName(),
-        import_name = expression.getImportName(),
-        needs_check = expression.mayRaiseException(BaseException),
+    module_name = expression.getModuleName()
+    needs_check = expression.mayRaiseException(BaseException)
+
+    emitLineNumberUpdateCode(emit, context)
+
+    emit(
+        """%s = PyImport_ImportModule("%s");""" % (
+            to_name,
+            module_name
+        )
+    )
+
+    getErrorExitCode(
+        check_name  = to_name,
+        needs_check = needs_check,
         emit        = emit,
         context     = context
     )
 
 
-def getImportModuleHardCode(to_name, module_name, import_name, needs_check,
-                            emit, context):
+def generateImportModuleNameHardCode(to_name, expression, emit, context):
+    module_name = expression.getModuleName()
+    import_name = expression.getImportName()
+    needs_check = expression.mayRaiseException(BaseException)
+
     if module_name == "sys":
         emit(
             """%s = PySys_GetObject( (char *)"%s" );""" % (
@@ -144,13 +157,27 @@ def getImportModuleHardCode(to_name, module_name, import_name, needs_check,
                 import_name
             )
         )
-    elif module_name in ("os", "__future__"):
+    elif module_name in ("os", "__future__", "importlib._bootstrap"):
+        emitLineNumberUpdateCode(emit, context)
+
         emit(
-             """%s = PyObject_GetAttrString(PyImport_ImportModule("%s"), "%s");""" % (
-                to_name,
-                module_name,
-                import_name
-            )
+             """\
+{
+    PyObject *module = PyImport_ImportModule("%(module_name)s");
+    if (likely( module != NULL ))
+    {
+        %(to_name)s = PyObject_GetAttr( module, %(import_name)s );
+    }
+    else
+    {
+        %(to_name)s = NULL;
+    }
+}
+""" % {
+                "to_name"     : to_name,
+                "module_name" : module_name,
+                "import_name" : context.getConstantCode(import_name)
+            }
         )
     else:
         assert False, module_name
@@ -177,7 +204,7 @@ def generateImportStarCode(statement, emit, context):
 
     res_name = context.getBoolResName()
 
-    if not context.hasLocalsDict():
+    if statement.getLocalsDictScope() is None:
         emit(
             "%s = IMPORT_MODULE_STAR( %s, true, %s );" % (
                 res_name,
@@ -188,13 +215,20 @@ def generateImportStarCode(statement, emit, context):
             )
         )
     else:
+        locals_dict_name = statement.getLocalsDictScope().getCodeName()
+
         emit(
-            "%s = IMPORT_MODULE_STAR( locals_dict, false, %s );" % (
-                res_name,
-                module_name
-            )
+            """
+%(res_name)s = IMPORT_MODULE_STAR( %(locals_dict)s, false, %(module_name)s );
+""" % {
+                "res_name"    : res_name,
+                "locals_dict" : locals_dict_name,
+                "module_name" : module_name
+            }
         )
 
+        context.addLocalsDictName(locals_dict_name)
+
     getReleaseCode(
         release_name = module_name,
         emit         = emit,
