@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,7 +28,7 @@ import math
 import re
 from logging import warning
 
-from nuitka.__past__ import (  # pylint: disable=redefined-builtin
+from nuitka.__past__ import (  # pylint: disable=I0021,redefined-builtin
     long,
     unicode,
     xrange
@@ -156,6 +156,8 @@ def namifyConstant(constant):
                 warning("Couldn't namify '%r'" % value)
 
                 return "list_" + _digest(repr(constant))
+    elif type(constant) is bytearray:
+        return "bytearray_" + _digest(repr(constant))
     elif type(constant) is xrange:
         return "xrange_%s" % (
             str(constant)[7 if str is bytes else 6:-1].replace(' ', "").replace(',', '_').replace('-', "neg")
