@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,8 +23,8 @@ in-place assignments, which have other operation variants.
 """
 
 from . import OperatorCodes
+from .CodeHelpers import generateChildExpressionsCode
 from .ErrorCodes import getErrorExitBoolCode, getErrorExitCode, getReleaseCode
-from .Helpers import generateChildExpressionsCode
 
 
 def generateOperationBinaryCode(to_name, expression, emit, context):
@@ -102,6 +102,8 @@ def getOperationCode(to_name, operator, arg_names, in_place, emit, context):
         helper = "BINARY_OPERATION_MUL"
     elif operator == "Mod":
         helper = "BINARY_OPERATION_REMAINDER"
+    elif operator == "Divmod":
+        helper = "BUILTIN_DIVMOD"
     elif len(arg_names) == 2:
         helper = "BINARY_OPERATION"
         prefix_args = (
