@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,8 +23,8 @@ happening in the C helper functions.
 
 """
 
+from .CodeHelpers import generateExpressionCode
 from .ErrorCodes import getErrorExitBoolCode, getReleaseCode, getReleaseCodes
-from .Helpers import generateExpressionCode
 
 
 def generatePrintValueCode(statement, emit, context):
