@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,17 +26,17 @@ from nuitka import Options
 from nuitka.Constants import isNumberConstant
 from nuitka.PythonVersions import python_version
 
+from .CodeHelpers import (
+    generateChildExpressionsCode,
+    generateExpressionCode,
+    generateExpressionsCode
+)
 from .ErrorCodes import (
     getErrorExitBoolCode,
     getErrorExitCode,
     getReleaseCode,
     getReleaseCodes
 )
-from .Helpers import (
-    generateChildExpressionsCode,
-    generateExpressionCode,
-    generateExpressionsCode
-)
 from .IndexCodes import (
     getIndexCode,
     getIndexValueCode,
