@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,12 +21,32 @@
 
 from nuitka.PythonVersions import python_version
 
+from .CodeHelpers import generateExpressionCode
 from .ErrorCodes import getErrorExitCode, getReleaseCode, getReleaseCodes
-from .Helpers import generateExpressionCode
 from .PythonAPICodes import generateCAPIObjectCode
 from .TupleCodes import getTupleCreationCode
 
 
+def generateBuiltinBytesCode(to_name, expression, emit, context):
+    encoding = expression.getEncoding()
+    errors = expression.getErrors()
+
+    generateCAPIObjectCode(
+        to_name    = to_name,
+        capi       = "BUILTIN_BYTES3",
+        arg_desc   = (
+            ("bytes_arg", expression.getValue()),
+            ("bytes_encoding", encoding),
+            ("bytes_errors", errors),
+        ),
+        may_raise  = expression.mayRaiseException(BaseException),
+        source_ref = expression.getCompatibleSourceReference(),
+        none_null  = True,
+        emit       = emit,
+        context    = context
+    )
+
+
 def generateBuiltinUnicodeCode(to_name, expression, emit, context):
     encoding = expression.getEncoding()
     errors = expression.getErrors()
@@ -60,7 +80,7 @@ def generateBuiltinUnicodeCode(to_name, expression, emit, context):
             source_ref = expression.getCompatibleSourceReference(),
             none_null  = True,
             emit       = emit,
-            context    = context,
+            context    = context
         )
 
 
