@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,8 +19,8 @@
 
 """
 
+from .CodeHelpers import generateExpressionCode
 from .ConstantCodes import getConstantAccess
-from .Helpers import generateExpressionCode
 from .PythonAPICodes import generateCAPIObjectCode
 
 
@@ -31,6 +31,7 @@ def _areConstants(expressions):
 
         if expression.isMutable():
             return False
+
     return True
 
 
@@ -48,7 +49,9 @@ def getTupleCreationCode(to_name, elements, emit, context):
         getConstantAccess(
             to_name  = to_name,
             constant = tuple(
-                element.getConstant() for element in elements
+                element.getConstant()
+                for element in
+                elements
             ),
             emit     = emit,
             context  = context
