@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,11 +20,25 @@
 """
 
 template_genfunc_yielder_decl_template = """\
+#if _NUITKA_EXPERIMENTAL_GENERATOR_GOTO
+static PyObject *%(function_identifier)s_context( struct Nuitka_GeneratorObject *generator, PyObject *yield_return_value );
+#else
 static void %(function_identifier)s_context( struct Nuitka_GeneratorObject *generator );
+#endif
 """
 
 template_genfunc_yielder_body_template = """
+#if _NUITKA_EXPERIMENTAL_GENERATOR_GOTO
+struct %(function_identifier)s_locals {
+%(function_local_types)s
+};
+#endif
+
+#if _NUITKA_EXPERIMENTAL_GENERATOR_GOTO
+static PyObject *%(function_identifier)s_context( struct Nuitka_GeneratorObject *generator, PyObject *yield_return_value )
+#else
 static void %(function_identifier)s_context( struct Nuitka_GeneratorObject *generator )
+#endif
 {
     CHECK_OBJECT( (PyObject *)generator );
     assert( Nuitka_Generator_Check( (PyObject *)generator ) );
@@ -32,6 +46,9 @@ static void %(function_identifier)s_context( struct Nuitka_GeneratorObject *gene
     // Local variable initialization
 %(function_var_inits)s
 
+    // Dispatch to yield based on return label index:
+%(function_dispatch)s
+
     // Actual function code.
 %(function_body)s
 
@@ -40,20 +57,38 @@ static void %(function_identifier)s_context( struct Nuitka_GeneratorObject *gene
 """
 
 template_generator_exception_exit = """\
+%(function_cleanup)s\
+#if _NUITKA_EXPERIMENTAL_GENERATOR_GOTO
+    return NULL;
+#else
     generator->m_yielded = NULL;
     return;
+#endif
 
     function_exception_exit:
+%(function_cleanup)s\
     assert( exception_type );
     RESTORE_ERROR_OCCURRED( exception_type, exception_value, exception_tb );
+
+#if _NUITKA_EXPERIMENTAL_GENERATOR_GOTO
+    return NULL;
+#else
     generator->m_yielded = NULL;
-    return;"""
+    return;
+#endif
+"""
 
 template_generator_noexception_exit = """\
     // Return statement need not be present.
+%(function_cleanup)s\
 
+#if _NUITKA_EXPERIMENTAL_GENERATOR_GOTO
+    return NULL;
+#else
     generator->m_yielded = NULL;
-    return;"""
+    return;
+#endif
+"""
 
 template_generator_return_exit = """\
     // The above won't return, but we need to make it clear to the compiler
@@ -75,8 +110,13 @@ template_generator_return_exit = """\
         Py_DECREF( tmp_return_value );
     }
 #endif
+
+#if _NUITKA_EXPERIMENTAL_GENERATOR_GOTO
+    return NULL;
+#else
     generator->m_yielded = NULL;
     return;
+#endif
 """
 
 template_generator_making = """\
