@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -196,30 +196,51 @@ if (unlikely( %(tmp_name)s == NULL ))
 }
 """
 
-template_read_maybe_local_unclear = """\
-%(tmp_name)s = PyDict_GetItem( %(locals_dict)s, %(var_name)s );
+template_read_locals_dict_with_fallback = """\
+%(to_name)s = PyDict_GetItem( %(locals_dict)s, %(var_name)s );
 
-if ( %(tmp_name)s == NULL )
+if ( %(to_name)s == NULL )
 {
 %(fallback)s
 }
 """
 
+template_read_locals_mapping_with_fallback = """\
+%(to_name)s = PyObject_GetItem( %(locals_dict)s, %(var_name)s );
+
+if ( %(to_name)s == NULL )
+{
+    if ( CHECK_AND_CLEAR_KEY_ERROR_OCCURRED() )
+    {
+%(fallback)s
+    }
+}
+"""
+
+
 template_del_global_unclear = """\
 %(res_name)s = PyDict_DelItem( (PyObject *)moduledict_%(module_identifier)s, %(var_name)s );
 if ( %(res_name)s == -1 ) CLEAR_ERROR_OCCURRED();
 """
 
 template_update_locals_dict_value = """\
-DICT_SYNC_FROM_VARIABLE(
-    %(dict_name)s,
-    %(var_name)s,
-    %(access_code)s
-);
+if ( %(test_code)s )
+{
+    UPDATE_STRING_DICT0( (PyDictObject *)%(dict_name)s, (Nuitka_StringObject *)%(var_name)s, %(access_code)s );
+}
+else
+{
+    int res = PyDict_DelItem( %(dict_name)s, %(var_name)s );
+
+    if ( res != 0 )
+    {
+        CLEAR_ERROR_OCCURRED();
+    }
+}
 """
 
 template_set_locals_dict_value = """\
-if ( %(access_code)s )
+if ( %(test_code)s )
 {
     int res = PyDict_SetItem(
         %(dict_name)s,
@@ -232,11 +253,44 @@ if ( %(access_code)s )
 """
 
 template_update_locals_mapping_value = """\
-%(tmp_name)s = MAPPING_SYNC_FROM_VARIABLE( %(mapping_name)s, %(var_name)s, %(access_code)s );
+if ( %(test_code)s )
+{
+    int res = PyObject_SetItem(
+        %(mapping_name)s,
+        %(var_name)s,
+        %(access_code)s
+    );
+
+    %(tmp_name)s = res == 0;
+}
+else
+{
+    PyObject *test_value = PyObject_GetItem(
+        %(mapping_name)s,
+        %(var_name)s
+    );
+
+    if ( test_value )
+    {
+        Py_DECREF( test_value );
+
+        int res = PyObject_DelItem(
+            %(mapping_name)s,
+            %(var_name)s
+        );
+
+        %(tmp_name)s = res == 0;
+    }
+    else
+    {
+        CLEAR_ERROR_OCCURRED();
+        %(tmp_name)s = true;
+    }
+}
 """
 
 template_set_locals_mapping_value = """\
-if %(check_code)s
+if ( %(test_code)s )
 {
     %(tmp_name)s = SET_SUBSCRIPT(
         %(mapping_name)s,
@@ -244,6 +298,10 @@ if %(check_code)s
         %(access_code)s
     );
 }
+else
+{
+    %(tmp_name)s = true;
+}
 """
 
 
