@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,6 +22,9 @@ MacOS, Windows, and Linux. Patches for other platforms are
 very welcome.
 """
 
+from __future__ import print_function
+
+import hashlib
 import marshal
 import os
 import shutil
@@ -45,12 +48,14 @@ from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
 from nuitka.tree.SourceReading import readSourceCodeFromFilename
 from nuitka.utils import Utils
+from nuitka.utils.AppDirs import getCacheDir
 from nuitka.utils.Execution import withEnvironmentPathAdded
 from nuitka.utils.FileOperations import (
     areSamePaths,
     deleteFile,
     getSubDirectories,
-    listDir
+    listDir,
+    makePath
 )
 from nuitka.utils.Timing import TimerReport
 
@@ -220,7 +225,7 @@ def _detectImports(command, user_provided, technical):
     if python_version >= 300:
         command += '\nprint("\\n".join(sorted("import " + module.__name__ + " # sourcefile " + ' \
                    'module.__file__ for module in sys.modules.values() if hasattr(module, "__file__") and ' \
-                   'module.__file__ != "<frozen>")), file = sys.stderr)'  # do not read it
+                   'module.__file__ not in (None, "<frozen>"))), file = sys.stderr)'  # do not read it
 
     reduced_path = [
         path_element
@@ -558,6 +563,12 @@ _detected_python_rpath = None
 ldd_result_cache = {}
 
 def _detectBinaryPathDLLsLinuxBSD(dll_filename):
+    # This is complex, as it also includes the caching mechanism
+    # pylint: disable=too-many-branches
+
+    if ldd_result_cache.get(dll_filename):
+        return ldd_result_cache[dll_filename]
+
     # Ask "ldd" about the libraries being used by the created binary, these
     # are the ones that interest us.
     result = set()
@@ -575,66 +586,72 @@ def _detectBinaryPathDLLsLinuxBSD(dll_filename):
                 os.path.dirname(sys.executable).encode("utf-8")
             )
 
-    if dll_filename not in ldd_result_cache:
-        with withEnvironmentPathAdded("LD_LIBRARY_PATH", _detected_python_rpath):
-            process = subprocess.Popen(
-                args   = [
-                    "ldd",
-                    dll_filename
-                ],
-                stdout = subprocess.PIPE,
-                stderr = subprocess.PIPE
-            )
+    with withEnvironmentPathAdded("LD_LIBRARY_PATH", _detected_python_rpath):
+        process = subprocess.Popen(
+            args   = [
+                "ldd",
+                dll_filename
+            ],
+            stdout = subprocess.PIPE,
+            stderr = subprocess.PIPE
+        )
 
-            stdout, _stderr = process.communicate()
+        stdout, _stderr = process.communicate()
 
-            for line in stdout.split(b"\n"):
-                if not line:
-                    continue
+        for line in stdout.split(b"\n"):
+            if not line:
+                continue
 
-                if b"=>" not in line:
-                    continue
+            if b"=>" not in line:
+                continue
 
-                part = line.split(b" => ", 2)[1]
+            part = line.split(b" => ", 2)[1]
 
-                if b"(" in part:
-                    filename = part[:part.rfind(b"(")-1]
-                else:
-                    filename = part
+            if b"(" in part:
+                filename = part[:part.rfind(b"(")-1]
+            else:
+                filename = part
 
-                if not filename:
-                    continue
+            if not filename:
+                continue
 
-                if python_version >= 300:
-                    filename = filename.decode("utf-8")
+            if python_version >= 300:
+                filename = filename.decode("utf-8")
 
-                # Sometimes might use stuff not found.
-                if filename == "not found":
-                    continue
+            # Sometimes might use stuff not found.
+            if filename == "not found":
+                continue
 
-                # Do not include kernel specific libraries.
-                if os.path.basename(filename).startswith(
-                        (
-                            "libc.so.",
-                            "libpthread.so.",
-                            "libm.so.",
-                            "libdl.so."
-                        )
-                    ):
-                    continue
+            # Do not include kernel specific libraries.
+            if os.path.basename(filename).startswith(
+                    (
+                        "libc.so.",
+                        "libpthread.so.",
+                        "libm.so.",
+                        "libdl.so."
+                    )
+                ):
+                continue
 
-                result.add(filename)
+            result.add(filename)
 
-        ldd_result_cache[dll_filename] = result
-    else:
-        result = ldd_result_cache[dll_filename]
+    # Allow plugins to prevent inclusion.
+    blocked = Plugins.removeDllDependencies(
+        dll_filename  = dll_filename,
+        dll_filenames = result
+    )
+
+    for to_remove in blocked:
+        result.discard(to_remove)
+
+    ldd_result_cache[dll_filename] = result
 
     sub_result = set(result)
 
     for sub_dll_filename in result:
-        sub_result.union(_detectBinaryPathDLLsLinuxBSD(sub_dll_filename))
+        sub_result = sub_result.union(_detectBinaryPathDLLsLinuxBSD(sub_dll_filename))
 
-    return result
+    return sub_result
 
 
 def _detectBinaryPathDLLsMacOS(original_dir, binary_filename):
@@ -678,32 +695,58 @@ def _detectBinaryPathDLLsMacOS(original_dir, binary_filename):
     return result
 
 
-def _makeBinaryPathPathDLLSearchEnv(package_name):
-    # Put the PYTHONPATH into the system "PATH", DLLs frequently live in
-    # the package directories.
-    env = os.environ.copy()
-    path = env.get("PATH","").split(os.pathsep)
+def _getCacheFilename(is_main_executable, source_dir, original_dir, binary_filename):
+    original_filename = os.path.join(
+        original_dir,
+        os.path.basename(binary_filename)
+    )
 
-    # Put the "Python.exe" first. At least for WinPython, they put the DLLs
-    # there.
-    path = [sys.prefix] + sys.path + path
-
-    if package_name is not None:
-        for element in sys.path:
-            candidate = os.path.join(element, package_name)
-
-            if os.path.isdir(candidate):
-                path.append(candidate)
+    original_filename = os.path.normcase(original_filename)
 
 
-    env["PATH"] = os.pathsep.join(path)
+    if is_main_executable:
+        # Normalize main program name for caching as well, but need to use the
+        # scons information to distinguish different Python and arches, and
+        # compilers, so we use different libs there.
+        hashed_value = open(os.path.join(source_dir, "scons-report.txt")).read()
+    else:
+        hashed_value = original_filename
 
-    return env
+    # Have different values for different Python major versions.
+    hashed_value += sys.version + sys.executable
+
+    if str is not bytes:
+        hashed_value = hashed_value.encode("utf8")
+
+    cache_dir = os.path.join(
+        getCacheDir(),
+        "library_deps",
+    )
+
+    makePath(cache_dir)
+
+    return os.path.join(
+        cache_dir,
+        hashlib.md5(hashed_value).hexdigest()
+    )
 
 
-def _detectBinaryPathDLLsWindows(original_dir, binary_filename, package_name):
+def _detectBinaryPathDLLsWindows(is_main_executable, source_dir, original_dir, binary_filename):
+    # This is complex, as it also includes the caching mechanism
+    # pylint: disable=too-many-branches
+
     result = set()
 
+    cache_filename = _getCacheFilename(is_main_executable, source_dir, original_dir, binary_filename)
+
+    if os.path.exists(cache_filename):
+        for line in open(cache_filename):
+            line = line.strip()
+
+            result.add(line)
+
+        return result
+
     depends_exe = getDependsExePath()
 
     # The search order by default prefers the system directory, where a
@@ -744,8 +787,7 @@ SxS
             "-pa1",
             "-ps1",
             binary_filename
-        ),
-        env = _makeBinaryPathPathDLLSearchEnv(package_name),
+        )
     )
 
     inside = False
@@ -848,10 +890,15 @@ SxS
     deleteFile(binary_filename + ".depends", must_exist = True)
     deleteFile(binary_filename + ".dwp", must_exist = True)
 
+    with open(cache_filename, 'w') as cache_file:
+        for dll_filename in result:
+            print(dll_filename, file = cache_file)
+
     return result
 
 
-def detectBinaryDLLs(original_filename, binary_filename, package_name):
+def detectBinaryDLLs(is_main_executable, source_dir, original_filename,
+                     binary_filename):
     """ Detect the DLLs used by a binary.
 
         Using "ldd" (Linux), "depends.exe" (Windows), or "otool" (MacOS) the list
@@ -866,9 +913,10 @@ def detectBinaryDLLs(original_filename, binary_filename, package_name):
     elif Utils.getOS() == "Windows":
         with TimerReport("Running depends.exe for %s took %%.2f seconds" % binary_filename):
             return _detectBinaryPathDLLsWindows(
-                original_dir    = os.path.dirname(original_filename),
-                binary_filename = binary_filename,
-                package_name    = package_name
+                is_main_executable = is_main_executable,
+                source_dir         = source_dir,
+                original_dir       = os.path.dirname(original_filename),
+                binary_filename    = binary_filename
             )
     elif Utils.getOS() == "Darwin":
         return _detectBinaryPathDLLsMacOS(
@@ -880,18 +928,20 @@ def detectBinaryDLLs(original_filename, binary_filename, package_name):
         assert False, Utils.getOS()
 
 
-def detectUsedDLLs(standalone_entry_points):
+def detectUsedDLLs(source_dir, standalone_entry_points):
     result = OrderedDict()
 
-    for original_filename, binary_filename, package_name in standalone_entry_points:
+    for count, (original_filename, binary_filename, _package_name) in enumerate(standalone_entry_points):
         used_dlls = detectBinaryDLLs(
-            original_filename = original_filename,
-            binary_filename   = binary_filename,
-            package_name      = package_name
+            is_main_executable = count == 0,
+            source_dir         = source_dir,
+            original_filename  = original_filename,
+            binary_filename    = binary_filename
         )
 
         for dll_filename in used_dlls:
-            # We want these to be absolute paths.
+            # We want these to be absolute paths. Solve that in the parts
+            # where detectBinaryDLLs is platform specific.
             assert os.path.isabs(dll_filename), dll_filename
 
             if dll_filename not in result:
@@ -988,14 +1038,15 @@ libraries that need to be removed."""
         assert retcode == 0, filename
 
 
-def copyUsedDLLs(dist_dir, standalone_entry_points):
+def copyUsedDLLs(source_dir, dist_dir, standalone_entry_points):
     # This is terribly complex, because we check the list of used DLLs
     # trying to avoid duplicates, and detecting errors with them not
     # being binary identical, so we can report them. And then of course
-    # we also need to handle OS specifics, pylint: disable=too-many-branches
+    # we also need to handle OS specifics.
+    # pylint: disable=too-many-branches,too-many-locals
 
 
-    used_dlls = detectUsedDLLs(standalone_entry_points)
+    used_dlls = detectUsedDLLs(source_dir, standalone_entry_points)
 
     # Fist make checks and remove some.
     for dll_filename1, sources1 in tuple(iterItems(used_dlls)):
