@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -30,7 +30,6 @@ the traces.
 
 """
 
-from nuitka import Options
 from nuitka.ModuleRegistry import getOwnerFromCodeName
 
 from .NodeBases import NodeBase, StatementChildrenHavingBase
@@ -38,7 +37,86 @@ from .NodeMakingHelpers import (
     makeStatementExpressionOnlyReplacementNode,
     makeStatementsSequenceReplacementNode
 )
-from .VariableRefNodes import ExpressionTempVariableRef, ExpressionVariableRef
+
+
+class StatementAssignmentVariableName(StatementChildrenHavingBase):
+    """ Precursor of StatementAssignmentVariable used during tree building phase
+
+    """
+
+    kind = "STATEMENT_ASSIGNMENT_VARIABLE_NAME"
+
+    named_children = (
+        "source",
+    )
+
+    def __init__(self, provider, variable_name, source, source_ref):
+        assert source is not None, source_ref
+
+        StatementChildrenHavingBase.__init__(
+            self,
+            values     = {
+                "source"       : source,
+            },
+            source_ref = source_ref
+        )
+
+        self.variable_name = variable_name
+        self.provider = provider
+
+        assert not provider.isExpressionOutlineBody(), source_ref
+
+    def getDetails(self):
+        return {
+            "variable_name" : self.variable_name,
+            "provider" : self.provider
+        }
+
+    def getVariableName(self):
+        return self.variable_name
+
+    def computeStatement(self, trace_collection):
+        # Only for abc, pylint: disable=no-self-use
+        assert False
+
+    getAssignSource = StatementChildrenHavingBase.childGetter(
+        "source"
+    )
+
+
+class StatementDelVariableName(NodeBase):
+    """ Precursor of StatementDelVariable used during tree building phase
+
+    """
+
+    kind = "STATEMENT_DEL_VARIABLE_NAME"
+
+    __slots__ = "variable_name", "provider", "tolerant"
+
+    def __init__(self, provider, variable_name, tolerant, source_ref):
+        NodeBase.__init__(
+            self,
+            source_ref = source_ref
+        )
+
+        self.variable_name = variable_name
+        self.provider = provider
+
+        self.tolerant = tolerant
+
+    def getDetails(self):
+        return {
+            "variable_name" : self.variable_name,
+            "provider" : self.provider,
+            "tolerant" : self.tolerant
+        }
+
+    def getVariableName(self):
+        return self.variable_name
+
+    def computeStatement(self, trace_collection):
+        # Only for abc, pylint: disable=no-self-use
+        assert False
 
 
 class StatementAssignmentVariable(StatementChildrenHavingBase):
@@ -59,22 +137,24 @@ class StatementAssignmentVariable(StatementChildrenHavingBase):
 
     named_children = (
         "source",
-        "variable_ref"
     )
 
     inplace_suspect = None
 
-    def __init__(self, variable_ref, source, source_ref):
-        assert variable_ref is not None, source_ref
+    def __init__(self, source, variable, source_ref, version = None):
         assert source is not None, source_ref
 
-        assert variable_ref.isTargetVariableRef()
+        if variable is not None:
+            if version is None:
+                version = variable.allocateTargetNumber()
+
+        self.variable = variable
+        self.variable_version = version
 
         StatementChildrenHavingBase.__init__(
             self,
             values     = {
                 "source"       : source,
-                "variable_ref" : variable_ref
             },
             source_ref = source_ref
         )
@@ -82,17 +162,59 @@ class StatementAssignmentVariable(StatementChildrenHavingBase):
         self.variable_trace = None
 
     def getDetail(self):
-        variable_ref = self.getTargetVariableRef()
-        variable = variable_ref.getVariable()
-
-        if variable is not None:
-            return "to variable %s" % variable
+        if self.variable is not None:
+            return "to variable %s" % self.variable
         else:
-            return "to variable %s" % self.getTargetVariableRef()
+            return "to variable %s" % self.variable_name
+
+    def getDetails(self):
+        return {
+            "variable"      : self.variable,
+        }
+
+    def getDetailsForDisplay(self):
+        return {
+            "variable_name" : self.getVariableName(),
+            "is_temp"       : self.variable.isTempVariable(),
+            "owner"         : self.variable.getOwner().getCodeName(),
+        }
+
+    @classmethod
+    def fromXML(cls, provider, source_ref, **args):
+        assert cls is StatementAssignmentVariable, cls
+
+        owner = getOwnerFromCodeName(args["owner"])
+
+        if args["is_temp"] == "True":
+            variable = owner.createTempVariable(args["variable_name"])
+        else:
+            variable = owner.getProvidedVariable(args["variable_name"])
+
+        del args["is_temp"]
+        del args["owner"]
+
+        version = variable.allocateTargetNumber()
+
+        return cls(
+            variable   = variable,
+            version    = version,
+            source_ref = source_ref,
+            **args
+        )
+
+    def makeClone(self):
+        if self.variable is not None:
+            version = self.variable.allocateTargetNumber()
+        else:
+            version = None
+
+        return StatementAssignmentVariable(
+            source     = self.getAssignSource().makeClone(),
+            variable   = self.variable,
+            version    = version,
+            source_ref = self.source_ref
+        )
 
-    getTargetVariableRef = StatementChildrenHavingBase.childGetter(
-        "variable_ref"
-    )
     getAssignSource = StatementChildrenHavingBase.childGetter(
         "source"
     )
@@ -100,6 +222,19 @@ class StatementAssignmentVariable(StatementChildrenHavingBase):
         "source"
     )
 
+    def getVariableName(self):
+        return self.variable.getName()
+
+    def getVariable(self):
+        return self.variable
+
+    def setVariable(self, variable):
+        self.variable = variable
+        self.variable_version = variable.allocateTargetNumber()
+
+    def getVariableVersion(self):
+        return self.variable_version
+
     def markAsInplaceSuspect(self):
         self.inplace_suspect = True
 
@@ -110,11 +245,10 @@ class StatementAssignmentVariable(StatementChildrenHavingBase):
         return self.getAssignSource().mayRaiseException(exception_type)
 
     def computeStatement(self, trace_collection):
-        # This is very complex stuff, pylint: disable=too-many-branches,too-many-return-statements
+        # This is very complex stuff, pylint: disable=too-many-branches
 
         # TODO: Way too ugly to have global trace kinds just here, and needs to
-        # be abstracted somehow. But for now we let it live here: pylint: disable=too-many-statements
-
+        # be abstracted somehow. But for now we let it live here.
         source = self.getAssignSource()
 
         if source.isExpressionSideEffects():
@@ -166,8 +300,7 @@ Side effects of assignments promoted to statements."""
             return result, "new_raise", """\
 Assignment raises exception in assigned value, removed assignment."""
 
-        variable_ref = self.getTargetVariableRef()
-        variable = variable_ref.getVariable()
+        variable = self.variable
 
         # Not allowed anymore at this point.
         assert variable is not None
@@ -181,8 +314,8 @@ Assignment raises exception in assigned value, removed assignment."""
         # supposed to be possible to eliminate. If we get that wrong, we are
         # doing it wrong.
         if not variable.isModuleVariable() and \
-             source.isExpressionVariableRef() and \
-             source.getVariable() == variable:
+           source.isExpressionVariableRef() and \
+           source.getVariable() is variable:
 
             # A variable access that has a side effect, must be preserved,
             # so it can e.g. raise an exception, otherwise we can be fully
@@ -213,81 +346,28 @@ Removed assignment of %s from itself which is known to be defined.""" % variable
             last_trace = variable.getMatchingAssignTrace(self)
 
             if last_trace is not None:
-                if variable.isLocalVariable() or variable.isTempVariable():
-                    if source.isCompileTimeConstant():
+                if source.isCompileTimeConstant():
+                    # Can safely forward propagate only non-mutable constants.
+                    if not source.isMutable():
+                        if not last_trace.getNameUsageCount():
+                            self.variable_trace.setReplacementNode(
+                                lambda _usage : source.makeClone()
+                            )
 
-                        # Can safely forward propagate only non-mutable constants.
-                        if not source.isMutable():
-
-                            if variable.isTempVariable() or \
-                               (not provider.isExpressionClassBody() and \
-                                not provider.isUnoptimized()
-                                ):
-
-                                if last_trace.hasDefiniteUsages():
-                                    self.variable_trace.setReplacementNode(
-                                        lambda usage : source.makeClone()
-                                    )
-                                    propagated = True
-                                else:
-                                    propagated = False
-
-                                if not last_trace.hasPotentialUsages() and not last_trace.hasNameUsages():
-                                    if not last_trace.getPrevious().isUninitTrace():
-                                        # TODO: We could well decide, if that's even necessary, but for now
-                                        # the "StatementDelVariable" is tasked with that.
-                                        result = StatementDelVariable(
-                                            variable_ref = self.getTargetVariableRef(),
-                                            tolerant     = True,
-                                            source_ref   = self.getSourceReference()
-                                        )
-                                    else:
-                                        result = None
-
-                                    return (
-                                        result,
-                                        "new_statements",
-                                        "Dropped %s assignment statement to '%s'." % (
-                                           "propagated" if propagated else "dead",
-                                           self.getTargetVariableRef().getVariableName()
-                                        )
-                                    )
+                            propagated = True
                         else:
-                            # Something might be possible still.
-
-                            pass
-                    elif False and Options.isExperimental() and \
-                        source.isExpressionFunctionCreation() and \
-                        source.getFunctionRef().getFunctionBody().isExpressionFunctionBody() and \
-                        not source.getDefaults() and  \
-                        not source.getKwDefaults() and \
-                        not source.getAnnotations():
-                        # TODO: These are very mutable, right?
-
-                        provider = self.getParentVariableProvider()
-
-                        if variable.isTempVariable() or \
-                           (not provider.isUnoptimized() and \
-                            not provider.isExpressionClassBody()):
-
-                            if last_trace.getDefiniteUsages() <= 1 and \
-                               not last_trace.hasPotentialUsages() and \
-                               not last_trace.hasNameUsages():
-
-                                if last_trace.getDefiniteUsages() == 1:
-                                    self.variable_trace.setReplacementNode(
-                                        lambda usage : source.makeClone()
-                                    )
-                                    propagated = True
-                                else:
-                                    propagated = False
+                            propagated = False
 
+                        if not variable.isModuleVariable():
+                            if not last_trace.hasPotentialUsages() and not last_trace.getNameUsageCount():
                                 if not last_trace.getPrevious().isUninitTrace():
-                                    # TODO: We could well decide, if that's even necessary.
+                                    # TODO: We could well decide, if that's even necessary, but for now
+                                    # the "StatementDelVariable" is tasked with that.
                                     result = StatementDelVariable(
-                                        variable_ref = self.getTargetVariableRef(),
-                                        tolerant     = True,
-                                        source_ref   = self.getSourceReference()
+                                        variable   = self.variable,
+                                        version    = self.variable_version,
+                                        tolerant   = True,
+                                        source_ref = self.getSourceReference()
                                     )
                                 else:
                                     result = None
@@ -297,12 +377,16 @@ Removed assignment of %s from itself which is known to be defined.""" % variable
                                     "new_statements",
                                     "Dropped %s assignment statement to '%s'." % (
                                        "propagated" if propagated else "dead",
-                                       self.getTargetVariableRef().getVariableName()
+                                       self.getVariableName()
                                     )
                                 )
                     else:
-                        # More cases thinkable.
+                        # Something might be possible still.
+
                         pass
+                else:
+                    # More cases thinkable.
+                    pass
 
         return self, None, None
 
@@ -317,7 +401,7 @@ Removed assignment of %s from itself which is known to be defined.""" % variable
             return None
 
 
-class StatementDelVariable(StatementChildrenHavingBase):
+class StatementDelVariable(NodeBase):
     """ Deleting a variable.
 
         All del forms that are not to attributes, slices, subscripts
@@ -334,59 +418,112 @@ class StatementDelVariable(StatementChildrenHavingBase):
     """
     kind = "STATEMENT_DEL_VARIABLE"
 
-    named_children = (
-        "variable_ref",
-    )
-
-    def __init__(self, variable_ref, tolerant, source_ref):
-        assert variable_ref is not None
-        assert variable_ref.isTargetVariableRef()
+    __slots__ = "variable", "variable_version", "variable_trace", "previous_version", "previous_trace", "tolerant"
 
+    def __init__(self, tolerant, source_ref, variable, version = None):
         if type(tolerant) is str:
             tolerant = tolerant == "True"
-
         assert tolerant is True or tolerant is False, repr(tolerant)
 
-        StatementChildrenHavingBase.__init__(
+        if variable is not None:
+            if version is None:
+                version = variable.allocateTargetNumber()
+
+        NodeBase.__init__(
             self,
-            values     = {
-                "variable_ref" : variable_ref
-            },
             source_ref = source_ref
         )
 
+        self.variable = variable
+        self.variable_version = version
+
+        self.previous_version = None
+
         self.variable_trace = None
         self.previous_trace = None
 
         self.tolerant = tolerant
 
     def getDetail(self):
-        variable_ref = self.getTargetVariableRef()
-        variable = variable_ref.getVariable()
-
-        if variable is not None:
-            return "to variable %s" % variable
+        if self.variable is not None:
+            return "to variable %s" % self.variable
         else:
-            return "to variable %s" % self.getTargetVariableRef()
+            return "to variable %s" % self.variable_name
 
     def getDetails(self):
         return {
+            "variable" : self.variable,
+            "version"  : self.variable_version,
             "tolerant" : self.tolerant
         }
 
+    def getDetailsForDisplay(self):
+        return {
+            "variable_name" : self.getVariableName(),
+            "is_temp"       : self.variable.isTempVariable(),
+            "owner"         : self.variable.getOwner().getCodeName(),
+            "tolerant"      : self.tolerant
+        }
+
+    @classmethod
+    def fromXML(cls, provider, source_ref, **args):
+        assert cls is StatementDelVariable, cls
+
+        owner = getOwnerFromCodeName(args["owner"])
+
+        if args["is_temp"] == "True":
+            variable = owner.createTempVariable(args["variable_name"])
+        else:
+            variable = owner.getProvidedVariable(args["variable_name"])
+
+        del args["is_temp"]
+        del args["owner"]
+
+        version = variable.allocateTargetNumber()
+        variable.version_number = max(variable.version_number, version)
+
+        return cls(
+            variable   = variable,
+            source_ref = source_ref,
+            **args
+        )
+
+    def makeClone(self):
+        if self.variable is not None:
+            version = self.variable.allocateTargetNumber()
+        else:
+            version = None
+
+        return StatementDelVariable(
+            variable   = self.variable,
+            version    = version,
+            tolerant   = self.tolerant,
+            source_ref = self.source_ref
+        )
+
     # TODO: Value propagation needs to make a difference based on this.
     def isTolerant(self):
         return self.tolerant
 
-    getTargetVariableRef = StatementChildrenHavingBase.childGetter(
-        "variable_ref"
-    )
+    def getVariableName(self):
+        return self.variable.getName()
+
+    def getVariable(self):
+        return self.variable
+
+    def setVariable(self, variable):
+        assert self.variable_version is None
+
+        self.variable = variable
+        self.variable_version = variable.allocateTargetNumber()
+
+    def getVariableVersion(self):
+        return self.variable_version
 
     def computeStatement(self, trace_collection):
-        variable_ref = self.getTargetVariableRef()
-        variable = variable_ref.getVariable()
+        variable = self.variable
 
-        self.previous_trace = trace_collection.getVariableCurrentTrace(variable)
+        self.previous_version, self.previous_trace = trace_collection.getVariableCurrentTraceVersion(variable)
 
         # First eliminate us entirely if we can.
         if self.tolerant and self.previous_trace.isUninitTrace():
@@ -394,7 +531,7 @@ class StatementDelVariable(StatementChildrenHavingBase):
                 None,
                 "new_statements",
                 "Removed tolerant 'del' statement of '%s' without effect." % (
-                    variable.getName(),
+                    self.getVariableName(),
                 )
             )
 
@@ -408,7 +545,8 @@ class StatementDelVariable(StatementChildrenHavingBase):
 
         # Record the deletion, needs to start a new version then.
         trace_collection.onVariableDel(
-            variable_ref = variable_ref
+            variable = variable,
+            version  = self.variable_version
         )
 
         trace_collection.onVariableContentEscapes(variable)
@@ -430,7 +568,7 @@ class StatementDelVariable(StatementChildrenHavingBase):
             return False
         else:
             if self.variable_trace is not None:
-                variable = self.getTargetVariableRef().getVariable()
+                variable = self.getVariable()
 
                 # Temporary variables deletions won't raise, just because we
                 # don't create them that way. We can avoid going through SSA in
@@ -457,7 +595,7 @@ class StatementReleaseVariable(NodeBase):
 
     kind = "STATEMENT_RELEASE_VARIABLE"
 
-    __slots__ = "variable", "variable_trace"
+    __slots__ = "variable", "variable_version", "variable_trace"
 
     def __init__(self, variable, source_ref):
         assert variable is not None, source_ref
@@ -468,6 +606,7 @@ class StatementReleaseVariable(NodeBase):
         )
 
         self.variable = variable
+        self.variable_version = None
 
         self.variable_trace = None
 
@@ -503,15 +642,13 @@ class StatementReleaseVariable(NodeBase):
         return self.variable
 
     def getVariableVersion(self):
-        return self.variable_trace.getVersion()
+        return self.variable_version
 
     def setVariable(self, variable):
         self.variable = variable
 
     def computeStatement(self, trace_collection):
-        self.variable_trace = trace_collection.onVariableRelease(
-            variable = self.variable
-        )
+        self.variable_version, self.variable_trace = trace_collection.getVariableCurrentTraceVersion(self.variable)
 
         if self.variable_trace.isUninitTrace():
             return (
@@ -540,120 +677,3 @@ class StatementReleaseVariable(NodeBase):
     def mayRaiseException(self, exception_type):
         # By default, __del__ is not allowed to raise an exception.
         return False
-
-
-class ExpressionTargetVariableRef(ExpressionVariableRef):
-    kind = "EXPRESSION_TARGET_VARIABLE_REF"
-
-    __slots__ = "variable_version",
-
-    # TODO: Remove default and correct argument order later.
-    def __init__(self, variable_name, source_ref, variable = None, version = None):
-        ExpressionVariableRef.__init__(self, variable_name, source_ref)
-
-        self.variable_version = version
-
-        # TODO: Remove setVariable, once not needed anymore and in-line to
-        # here.
-        if variable is not None:
-            self.setVariable(variable)
-            assert variable.getName() == variable_name
-
-    def getDetailsForDisplay(self):
-        result = {
-            "variable_name" : self.variable_name,
-            "version"       : self.variable_version,
-        }
-
-        if self.variable is not None:
-            result["owner"] = self.variable.getOwner().getCodeName()
-
-        return result
-
-    @classmethod
-    def fromXML(cls, provider, source_ref, **args):
-        assert cls is ExpressionTargetVariableRef, cls
-
-        owner = getOwnerFromCodeName(args["owner"])
-        assert owner is not None, args["owner"]
-
-        variable = owner.getProvidedVariable(args["variable_name"])
-
-        return cls(
-            variable_name = variable.getName(),
-            variable      = variable,
-            version       = int(args["version"]),
-            source_ref    = source_ref
-        )
-
-    def computeExpression(self, trace_collection):
-        assert False, self.parent
-
-    @staticmethod
-    def isTargetVariableRef():
-        return True
-
-    def getVariableVersion(self):
-        assert self.variable_version is not None, self
-
-        return self.variable_version
-
-    def setVariable(self, variable):
-        ExpressionVariableRef.setVariable(self, variable)
-
-        self.variable_version = variable.allocateTargetNumber()
-        assert self.variable_version is not None
-
-
-class ExpressionTargetTempVariableRef(ExpressionTempVariableRef):
-    kind = "EXPRESSION_TARGET_TEMP_VARIABLE_REF"
-
-    __slots__ = "variable_version",
-
-    def __init__(self, variable, source_ref, version = None):
-        ExpressionTempVariableRef.__init__(self, variable, source_ref)
-
-        if version is None:
-            version = variable.allocateTargetNumber()
-
-        self.variable_version = version
-
-    def getDetailsForDisplay(self):
-        return {
-            "temp_name" : self.variable.getName(),
-            "version"   : self.variable_version,
-            "owner"     : self.variable.getOwner().getCodeName()
-        }
-
-    @classmethod
-    def fromXML(cls, provider, source_ref, **args):
-        assert cls is ExpressionTargetTempVariableRef, cls
-
-        owner = getOwnerFromCodeName(args["owner"])
-        variable = owner.createTempVariable(args["temp_name"])
-        version = int(args["version"])
-
-        variable.version_number = max(variable.version_number, version)
-
-        return cls(
-            variable   = variable,
-            version    = version,
-            source_ref = source_ref
-        )
-
-
-    def computeExpression(self, trace_collection):
-        assert False, self.parent
-
-    @staticmethod
-    def isTargetVariableRef():
-        return True
-
-    def getVariableVersion(self):
-        return self.variable_version
-
-    # Python3 only, it updates temporary variables that are closure variables.
-    def setVariable(self, variable):
-        ExpressionTempVariableRef.setVariable(self, variable)
-
-        self.variable_version = self.variable.allocateTargetNumber()
