@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,9 +24,7 @@ whose implementation lives here. The creation itself also lives here.
 
 from .Checkers import checkStatementsSequenceOrNone
 from .ExpressionBases import ExpressionChildrenHavingBase
-from .FunctionNodes import ExpressionFunctionBodyBase
-from .IndicatorMixins import MarkLocalsDictIndicatorMixin
-from .NodeBases import ChildrenHavingMixin
+from .FunctionNodes import ExpressionFunctionEntryPointBase
 
 
 class ExpressionMakeAsyncgenObject(ExpressionChildrenHavingBase):
@@ -70,10 +68,12 @@ class ExpressionMakeAsyncgenObject(ExpressionChildrenHavingBase):
         self.variable_closure_traces = []
 
         for closure_variable in self.getAsyncgenRef().getFunctionBody().getClosureVariables():
-            trace = trace_collection.getVariableCurrentTrace(closure_variable)
+            version, trace = trace_collection.getVariableCurrentTraceVersion(closure_variable)
             trace.addClosureUsage()
 
-            self.variable_closure_traces.append(trace)
+            self.variable_closure_traces.append(
+                (closure_variable, version, trace)
+            )
 
         # TODO: Asyncgen body may know something too.
         return self, None, None
@@ -85,16 +85,10 @@ class ExpressionMakeAsyncgenObject(ExpressionChildrenHavingBase):
         return False
 
     def getClosureVariableVersions(self):
-        return [
-            (trace.getVariable(), trace.getVersion())
-            for trace in self.variable_closure_traces
-        ]
+        return self.variable_closure_traces
 
 
-class ExpressionAsyncgenObjectBody(MarkLocalsDictIndicatorMixin,
-                                   ExpressionFunctionBodyBase):
-    # We really want these many ancestors, as per design, we add properties via
-    # base class mix-ins a lot, pylint: disable=R0901
+class ExpressionAsyncgenObjectBody(ExpressionFunctionEntryPointBase):
     kind = "EXPRESSION_ASYNCGEN_OBJECT_BODY"
 
     named_children = (
@@ -109,29 +103,20 @@ class ExpressionAsyncgenObjectBody(MarkLocalsDictIndicatorMixin,
     qualname_setup = None
 
     def __init__(self, provider, name, flags, source_ref):
-        while provider.isExpressionOutlineBody():
-            provider = provider.getParentVariableProvider()
-
-        ExpressionFunctionBodyBase.__init__(
+        ExpressionFunctionEntryPointBase.__init__(
             self,
             provider    = provider,
             name        = name,
             code_prefix = "asyncgen",
-            is_class    = False,
             flags       = flags,
             source_ref  = source_ref
         )
 
-        MarkLocalsDictIndicatorMixin.__init__(self)
-
         self.needs_generator_return_exit = False
 
     def getFunctionName(self):
         return self.name
 
-    def getFunctionQualname(self):
-        return self.getParentVariableProvider().getFunctionQualname()
-
     def markAsNeedsGeneratorReturnHandling(self, value):
         self.needs_generator_return_exit = max(
             self.needs_generator_return_exit,
@@ -151,6 +136,3 @@ class ExpressionAsyncgenObjectBody(MarkLocalsDictIndicatorMixin,
     @staticmethod
     def isUnoptimized():
         return False
-
-    getBody = ChildrenHavingMixin.childGetter("body")
-    setBody = ChildrenHavingMixin.childSetter("body")
