@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,16 +19,13 @@
 
 """
 
-from nuitka.nodes.DictionaryNodes import (
-    ExpressionKeyValuePair,
-    ExpressionMakeDict
-)
-from nuitka.nodes.NodeMakingHelpers import wrapExpressionWithNodeSideEffects
 from nuitka.optimizations.BuiltinOptimization import builtin_dict_spec
 
 from .BuiltinIteratorNodes import ExpressionBuiltinIter1
 from .ConstantRefNodes import makeConstantRefNode
+from .DictionaryNodes import ExpressionKeyValuePair, ExpressionMakeDict
 from .ExpressionBases import ExpressionChildrenHavingBase
+from .NodeMakingHelpers import wrapExpressionWithNodeSideEffects
 
 
 class ExpressionBuiltinDict(ExpressionChildrenHavingBase):
