@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,7 +25,7 @@ about performance critical.
 from nuitka.optimizations import BuiltinOptimization
 from nuitka.PythonVersions import python_version
 
-from .NodeBases import (
+from .ExpressionBases import (
     ExpressionBuiltinSingleArgBase,
     ExpressionChildrenHavingBase
 )
@@ -76,7 +76,7 @@ class ExpressionBuiltinFormat(ExpressionChildrenHavingBase):
                 return (
                     value,
                     "new_expression", """\
-Removed useless 'format' on %s value.""" % value.getTypeShape().getTypeName()
+Removed useless 'format' on '%s' value.""" % value.getTypeShape().getTypeName()
                 )
 
         return self, None, None
