@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -46,6 +46,3 @@ class ExpressionBuiltinVars(ExpressionChildrenHavingBase):
     def computeExpression(self, trace_collection):
         # TODO: Should be possible to predict this.
         return self, None, None
-
-    def mayBeNone(self):
-        return False
