@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -68,10 +68,12 @@ class ExpressionMakeCoroutineObject(ExpressionChildrenHavingBase):
         self.variable_closure_traces = []
 
         for closure_variable in self.getCoroutineRef().getFunctionBody().getClosureVariables():
-            trace = trace_collection.getVariableCurrentTrace(closure_variable)
+            version, trace = trace_collection.getVariableCurrentTraceVersion(closure_variable)
             trace.addClosureUsage()
 
-            self.variable_closure_traces.append(trace)
+            self.variable_closure_traces.append(
+                (closure_variable, version, trace)
+            )
 
         # TODO: Coroutine body may know something too.
         return self, None, None
@@ -83,10 +85,7 @@ class ExpressionMakeCoroutineObject(ExpressionChildrenHavingBase):
         return False
 
     def getClosureVariableVersions(self):
-        return [
-            (trace.getVariable(), trace.getVersion())
-            for trace in self.variable_closure_traces
-        ]
+        return self.variable_closure_traces
 
 
 class ExpressionCoroutineObjectBody(ExpressionFunctionEntryPointBase):
