@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -76,9 +76,6 @@ if python_version >= 300:
                 source_ref  = source_ref
             )
 
-        def needsLocalsDict(self):
-            return True
-
         def computeExpression(self, trace_collection):
             # TODO: Attempt for constant values to do it.
             return self, None, None
@@ -114,9 +111,6 @@ if python_version < 300:
                 source_ref  = source_ref
             )
 
-        def needsLocalsDict(self):
-            return True
-
         def computeExpressionDrop(self, statement, trace_collection):
             # In this case, the copy-back must be done and will only be done
             # correctly by the code for exec statements.
@@ -166,9 +160,6 @@ class StatementExec(StatementChildrenHavingBase):
     getGlobals = StatementChildrenHavingBase.childGetter("globals")
     getLocals = StatementChildrenHavingBase.childGetter("locals")
 
-    def needsLocalsDict(self):
-        return self.getLocals().mayBeNone()
-
     def computeStatement(self, trace_collection):
         trace_collection.onExpression(
             expression = self.getSourceCode()
@@ -298,9 +289,9 @@ class StatementLocalsDictSync(StatementChildrenHavingBase):
         if self.getParentVariableProvider().isCompiledPythonModule():
             return None, "new_statements", "Removed sync back to locals without locals."
 
-        self.previous_traces = trace_collection.onLocalsUsage()
+        self.previous_traces = trace_collection.onLocalsUsage(self.getParentVariableProvider())
         trace_collection.removeAllKnowledge()
-        self.variable_traces = trace_collection.onLocalsUsage()
+        self.variable_traces = trace_collection.onLocalsUsage(self.getParentVariableProvider())
 
         return self, None, None
 
