@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -255,7 +255,7 @@ class StatementExpressionOnly(StatementChildrenHavingBase):
 class StatementPreserveFrameException(NodeBase):
     kind = "STATEMENT_PRESERVE_FRAME_EXCEPTION"
 
-    __slots__ = "preserver_id",
+    __slots__ = ("preserver_id",)
 
     def __init__(self, preserver_id, source_ref):
         NodeBase.__init__(
@@ -296,7 +296,7 @@ class StatementPreserveFrameException(NodeBase):
 class StatementRestoreFrameException(NodeBase):
     kind = "STATEMENT_RESTORE_FRAME_EXCEPTION"
 
-    __slots__ = "preserver_id",
+    __slots__ = ("preserver_id",)
 
     def __init__(self, preserver_id, source_ref):
         NodeBase.__init__(
