@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,8 +24,8 @@ and its expressions, changing the meaning of course dramatically.
 
 from nuitka import Builtins, Variables
 from nuitka.ModuleRegistry import getOwnerFromCodeName
+from nuitka.PythonVersions import python_version
 
-from .ConstantRefNodes import makeConstantRefNode
 from .DictionaryNodes import (
     ExpressionDictOperationGet,
     ExpressionDictOperationIn,
@@ -34,13 +34,77 @@ from .DictionaryNodes import (
     StatementDictOperationSet
 )
 from .ExpressionBases import ExpressionBase
+from .ModuleAttributeNodes import (
+    ExpressionModuleAttributeLoaderRef,
+    ExpressionModuleAttributeNameRef,
+    ExpressionModuleAttributePackageRef,
+    ExpressionModuleAttributeSpecRef
+)
+from .NodeMakingHelpers import makeRaiseExceptionReplacementExpression
 from .shapes.StandardShapes import ShapeUnknown
 
 
+class ExpressionVariableNameRef(ExpressionBase):
+    """ These are used before the actual variable object is known from VariableClosure.
+
+    """
+
+    kind = "EXPRESSION_VARIABLE_NAME_REF"
+
+    __slots__ = "variable_name", "provider"
+
+    def __init__(self, provider, variable_name, source_ref):
+        assert not provider.isExpressionOutlineBody(), source_ref
+
+        ExpressionBase.__init__(
+            self,
+            source_ref = source_ref
+        )
+
+        self.variable_name = variable_name
+
+        self.provider = provider
+
+    @staticmethod
+    def isExpressionVariableNameRef():
+        return True
+
+    def getDetails(self):
+        return {
+            "variable_name" : self.variable_name,
+            "provider"      : self.provider
+        }
+
+    def getVariableName(self):
+        return self.variable_name
+
+    def computeExpressionRaw(self, trace_collection):
+        return self, None, None
+
+    @staticmethod
+    def needsFallback():
+        return True
+
+
+class ExpressionVariableLocalNameRef(ExpressionVariableNameRef):
+    """ These are used before the actual variable object is known from VariableClosure.
+
+        The special thing about this as opposed to ExpressionVariableNameRef is that
+        these must remain local names and cannot fallback to outside scopes. This is
+        used for __annotations__.
+
+    """
+    kind = "EXPRESSION_VARIABLE_LOCAL_NAME_REF"
+
+    @staticmethod
+    def needsFallback():
+        return False
+
+
 class ExpressionVariableRefBase(ExpressionBase):
     # Base classes can be abstract, pylint: disable=abstract-method
 
-    __slots__ = "variable", "variable_trace"
+    __slots__ = "variable", "variable_version", "variable_trace"
 
     def __init__(self, variable, source_ref):
         ExpressionBase.__init__(
@@ -50,6 +114,7 @@ class ExpressionVariableRefBase(ExpressionBase):
 
         self.variable = variable
         self.variable_trace = None
+        self.variable_version = None
 
     def getVariableName(self):
         return self.variable.getName()
@@ -58,17 +123,19 @@ class ExpressionVariableRefBase(ExpressionBase):
         return self.variable
 
     def getVariableVersion(self):
-        return self.variable_trace.getVersion()
+        return self.variable_version
+
+    def getVariableTrace(self):
+        return self.variable_trace
 
 
 class ExpressionVariableRef(ExpressionVariableRefBase):
     kind = "EXPRESSION_VARIABLE_REF"
 
-    __slots__ = "variable_name",
+    __slots__ = ()
 
-    def __init__(self, variable_name, source_ref, variable = None):
-        if variable is not None:
-            assert variable.getName() == variable_name
+    def __init__(self, variable, source_ref):
+        assert variable is not None
 
         ExpressionVariableRefBase.__init__(
             self,
@@ -76,42 +143,27 @@ class ExpressionVariableRef(ExpressionVariableRefBase):
             source_ref = source_ref
         )
 
-        self.variable_name = variable_name
-
     def getDetails(self):
-        if self.variable is None:
-            return {
-                "variable_name" : self.variable_name
-            }
-        else:
-            return {
-                "variable_name" : self.variable_name,
-                "variable"      : self.variable
-            }
+        return {
+            "variable"      : self.variable
+        }
 
     def getDetailsForDisplay(self):
-        if self.variable is None:
-            return {
-                "variable_name" : self.variable_name
-            }
-        else:
-            return {
-                "variable_name" : self.variable_name,
-                "owner"         : self.variable.getOwner().getCodeName()
-            }
+        return {
+            "variable_name" : self.variable.getName(),
+            "owner"         : self.variable.getOwner().getCodeName()
+        }
 
     @classmethod
     def fromXML(cls, provider, source_ref, **args):
         assert cls is ExpressionVariableRef, cls
 
         owner = getOwnerFromCodeName(args["owner"])
-
         variable = owner.getProvidedVariable(args["variable_name"])
 
         return cls(
-            variable_name = variable.getName(),
-            variable      = variable,
-            source_ref    = source_ref
+            variable   = variable,
+            source_ref = source_ref
         )
 
     def getDetail(self):
@@ -145,7 +197,7 @@ class ExpressionVariableRef(ExpressionVariableRefBase):
         variable = self.variable
         assert variable is not None
 
-        self.variable_trace = trace_collection.getVariableCurrentTrace(
+        self.variable_version, self.variable_trace = trace_collection.getVariableCurrentTraceVersion(
             variable = variable
         )
 
@@ -170,55 +222,73 @@ class ExpressionVariableRef(ExpressionVariableRefBase):
                 BaseException
             )
 
+        if variable.isModuleVariable() and \
+           variable.hasDefiniteWrites() is False:
+            variable_name = self.variable.getName()
 
-        if variable.isModuleVariable() \
-            and variable.hasDefiniteWrites() is False:
-            if self.variable_name in Builtins.builtin_exception_names:
+            if variable_name in Builtins.builtin_exception_names:
                 from .BuiltinRefNodes import ExpressionBuiltinExceptionRef
 
                 new_node = ExpressionBuiltinExceptionRef(
-                    exception_name = self.variable_name,
+                    exception_name = self.variable.getName(),
                     source_ref     = self.getSourceReference()
                 )
 
                 change_tags = "new_builtin_ref"
                 change_desc = """\
 Module variable '%s' found to be built-in exception reference.""" % (
-                    self.variable_name
+                    variable_name
                 )
-            elif self.variable_name in Builtins.builtin_names and \
-                 self.variable_name != "pow":
-                from .BuiltinRefNodes import ExpressionBuiltinRef
+            elif variable_name in Builtins.builtin_names and \
+                 variable_name != "pow":
+                from .BuiltinRefNodes import makeExpressionBuiltinRef
 
-                new_node = ExpressionBuiltinRef(
-                    builtin_name = self.variable_name,
+                new_node = makeExpressionBuiltinRef(
+                    builtin_name = variable_name,
                     source_ref   = self.getSourceReference()
                 )
 
                 change_tags = "new_builtin_ref"
                 change_desc = """\
 Module variable '%s' found to be built-in reference.""" % (
-                    self.variable_name
+                    variable_name
                 )
-            elif self.variable_name == "__name__":
-                new_node = makeConstantRefNode(
-                    constant   = variable.getOwner().getParentModule().\
-                                   getFullName(),
+            elif variable_name == "__name__":
+                new_node = ExpressionModuleAttributeNameRef(
+                    module     = variable.getOwner(),
                     source_ref = self.getSourceReference()
                 )
 
-                change_tags = "new_constant"
+                change_tags = "new_expression"
                 change_desc = """\
-Replaced read-only module attribute '__name__' with constant value."""
-            elif self.variable_name == "__package__":
-                new_node = makeConstantRefNode(
-                    constant   = variable.getOwner().getPackage(),
+Replaced read-only module attribute '__name__' with module attribute reference."""
+            elif variable_name == "__package__":
+                new_node = ExpressionModuleAttributePackageRef(
+                    module     = variable.getOwner(),
                     source_ref = self.getSourceReference()
                 )
 
-                change_tags = "new_constant"
+                change_tags = "new_expression"
                 change_desc = """\
-Replaced read-only module attribute '__package__' with constant value."""
+Replaced read-only module attribute '__package__' with module attribute reference."""
+            elif variable_name == "__loader__" and python_version >= 330:
+                new_node = ExpressionModuleAttributeLoaderRef(
+                    module     = variable.getOwner(),
+                    source_ref = self.getSourceReference()
+                )
+
+                change_tags = "new_expression"
+                change_desc = """\
+Replaced read-only module attribute '__loader__' with module attribute reference."""
+            elif variable_name == "__spec__" and python_version >= 340:
+                new_node = ExpressionModuleAttributeSpecRef(
+                    module     = variable.getOwner(),
+                    source_ref = self.getSourceReference()
+                )
+
+                change_tags = "new_expression"
+                change_desc = """\
+Replaced read-only module attribute '__spec__' with module attribute reference."""
             else:
                 self.variable_trace.addUsage()
 
@@ -231,6 +301,20 @@ Replaced read-only module attribute '__package__' with constant value."""
 
         self.variable_trace.addUsage()
 
+        if self.variable_trace.mustNotHaveValue():
+            assert self.variable.isLocalVariable(), self.variable
+
+            variable_name = self.variable.getName()
+
+            result = makeRaiseExceptionReplacementExpression(
+                expression      = self,
+                exception_type  = "UnboundLocalError",
+                exception_value = """\
+local variable '%s' referenced before assignment""" % variable_name
+            )
+
+            return result, "new_raise", "Variable access of not initialized variable '%s'" % variable_name
+
         return self, None, None
 
     def computeExpressionCall(self, call_node, call_args, call_kw,
@@ -241,10 +325,10 @@ Replaced read-only module attribute '__package__' with constant value."""
         trace_collection.onControlFlowEscape(self)
 
         if not Variables.complete and \
-           self.variable_name in ("dir", "eval", "exec", "execfile", "locals", "vars") and \
+           self.variable.getName() in ("dir", "eval", "exec", "execfile", "locals", "vars") and \
            self.variable.isModuleVariable():
             # Just inform the collection that all escaped.
-            trace_collection.onLocalsUsage()
+            trace_collection.onLocalsUsage(self.getParentVariableProvider())
 
         return call_node, None, None
 
@@ -396,7 +480,7 @@ class ExpressionTempVariableRef(ExpressionVariableRefBase):
     def getDetailsForDisplay(self):
         return {
             "temp_name" : self.variable.getName(),
-            "owner" : self.variable.getOwner().getCodeName()
+            "owner"     : self.variable.getOwner().getCodeName()
         }
 
     def getDetails(self):
@@ -433,7 +517,7 @@ class ExpressionTempVariableRef(ExpressionVariableRefBase):
             return ShapeUnknown
 
     def computeExpressionRaw(self, trace_collection):
-        self.variable_trace = trace_collection.getVariableCurrentTrace(
+        self.variable_version, self.variable_trace = trace_collection.getVariableCurrentTraceVersion(
             variable = self.variable
         )
 
@@ -484,11 +568,11 @@ class ExpressionTempVariableRef(ExpressionVariableRefBase):
         trace_collection.onVariableContentEscapes(self.variable)
 
     def mayHaveSideEffects(self):
-        # Can't happen
+        # Can't happen with temporary variables.
         return False
 
     def mayRaiseException(self, exception_type):
-        # Can't happen
+        # Can't happen with temporary variables.
         return False
 
     def mayRaiseExceptionImportName(self, exception_type, import_name):
@@ -507,115 +591,3 @@ class ExpressionTempVariableRef(ExpressionVariableRefBase):
     def isKnownToBeIterableAtMax(self, count):
         # TODO: See through the variable current trace.
         return None
-
-    # Python3 only, it updates temporary variables that are closure variables.
-    def setVariable(self, variable):
-        self.variable = variable
-
-
-class ExpressionLocalsVariableRef(ExpressionBase):
-    kind = "EXPRESSION_LOCALS_VARIABLE_REF"
-
-    __slots__ = "variable_name", "fallback_variable", "variable_trace"
-
-    def __init__(self, variable_name, fallback_variable, source_ref):
-        self.variable_name = variable_name
-
-        ExpressionBase.__init__(
-            self,
-            source_ref = source_ref
-        )
-
-        self.fallback_variable = fallback_variable
-
-        self.variable_trace = None
-
-    def getDetails(self):
-        return {
-            "variable_name" : self.variable_name,
-        }
-
-    def getVariableName(self):
-        return self.variable_name
-
-    def getFallbackVariable(self):
-        return self.fallback_variable
-
-    def getFallbackVariableVersion(self):
-        return self.variable_trace.getVersion()
-
-    def computeExpressionRaw(self, trace_collection):
-        # TODO: Use dictionary tracings for locals dict
-
-        self.variable_trace = trace_collection.getVariableCurrentTrace(
-            variable = self.fallback_variable
-        )
-
-        if self.fallback_variable.isModuleVariable() and \
-           self.fallback_variable.hasDefiniteWrites() is False:
-            if self.variable_name in Builtins.builtin_exception_names:
-                from .BuiltinRefNodes import ExpressionBuiltinExceptionRef
-
-                new_node = ExpressionBuiltinExceptionRef(
-                    exception_name = self.variable_name,
-                    source_ref     = self.getSourceReference()
-                )
-
-                change_tags = "new_builtin_ref"
-                change_desc = """\
-Module variable '%s' found to be built-in exception reference.""" % (
-                    self.variable_name
-                )
-            elif self.variable_name in Builtins.builtin_names and \
-                 self.variable_name != "pow":
-                from .BuiltinRefNodes import ExpressionBuiltinRef
-
-                new_node = ExpressionBuiltinRef(
-                    builtin_name = self.variable_name,
-                    source_ref   = self.getSourceReference()
-                )
-
-                change_tags = "new_builtin_ref"
-                change_desc = """\
-Module variable '%s' found to be built-in reference.""" % (
-                    self.variable_name
-                )
-            elif self.variable_name == "__name__":
-                new_node = makeConstantRefNode(
-                    constant   = self.getParentModule().getFullName(),
-                    source_ref = self.getSourceReference()
-                )
-
-                change_tags = "new_constant"
-                change_desc = """\
-Replaced read-only module attribute '__name__' with constant value."""
-            elif self.variable_name == "__package__":
-                new_node = makeConstantRefNode(
-                    constant   = self.getParentModule().getPackage(),
-                    source_ref = self.getSourceReference()
-                )
-
-                change_tags = "new_constant"
-                change_desc = """\
-Replaced read-only module attribute '__package__' with constant value."""
-            else:
-                # Probably should give a warning once about it.
-                new_node = self
-                change_tags = None
-                change_desc = None
-
-            if new_node is not self:
-                return new_node, change_tags, change_desc
-
-        # TODO: This could be way more specific surely, using dictionary tracing
-        if not self.variable_trace.mustHaveValue():
-            trace_collection.onExceptionRaiseExit(
-                BaseException
-            )
-
-        self.variable_trace.addUsage()
-
-        return self, None, None
-
-    def mayRaiseException(self, exception_type):
-        return self.variable_trace is None or not self.variable_trace.mustHaveValue()
