@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -46,6 +46,14 @@ class ShapeUnknown(ShapeBase):
     def hasShapeSlotInt():
         return None
 
+    @staticmethod
+    def hasShapeSlotLong():
+        return None
+
+    @staticmethod
+    def hasShapeSlotFloat():
+        return None
+
     @staticmethod
     def hasShapeSlotIter():
         return None
@@ -96,7 +104,7 @@ class ShapeLargeConstantValue(object):
 
 
 class ShapeLargeConstantValuePredictable(ShapeLargeConstantValue):
-    __slots__ = "predictor",
+    __slots__ = ("predictor",)
 
     def __init__(self, size, predictor, shape):
         ShapeLargeConstantValue.__init__(self, size, shape)
@@ -113,6 +121,14 @@ class ShapeIterator(ShapeBase):
     def hasShapeSlotInt():
         return None
 
+    @staticmethod
+    def hasShapeSlotLong():
+        return None
+
+    @staticmethod
+    def hasShapeSlotFloat():
+        return None
+
     @staticmethod
     def hasShapeSlotIter():
         return True
