@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,6 +20,7 @@
 """
 
 import marshal
+from logging import debug
 
 from nuitka.importing.ImportCache import replaceImportedModule
 from nuitka.ModuleRegistry import replaceRootModule
@@ -32,9 +33,17 @@ def demoteCompiledModuleToBytecode(module):
     """ Demote a compiled module to uncompiled (bytecode).
 
     """
+
     full_name = module.getFullName()
     filename = module.getCompileTimeFilename()
 
+    debug(
+        "Demoting module '%s' to bytecode from '%s'.",
+        full_name,
+        filename
+    )
+
+
     source_code = readSourceCodeFromFilename(full_name, filename)
 
     source_code = Plugins.onFrozenModuleSourceCode(
@@ -43,7 +52,7 @@ def demoteCompiledModuleToBytecode(module):
         source_code = source_code
     )
 
-    bytecode = compile(source_code, filename, "exec")
+    bytecode = compile(source_code, filename, "exec", dont_inherit = True)
 
     bytecode = Plugins.onFrozenModuleBytecode(
         module_name = full_name,
@@ -52,7 +61,7 @@ def demoteCompiledModuleToBytecode(module):
     )
 
     uncompiled_module = makeUncompiledPythonModule(
-        module_name   = module.getFullName(),
+        module_name   = full_name,
         filename      = filename,
         bytecode      = marshal.dumps(bytecode),
         is_package    = module.isCompiledPythonPackage(),
