@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,36 +29,18 @@ import sys
 import tempfile
 from contextlib import contextmanager
 
-from nuitka.utils.FileOperations import removeDirectory
+from nuitka.Tracing import my_print
+from nuitka.utils.AppDirs import getAppDir, getCacheDir
+from nuitka.utils.Execution import check_output
+from nuitka.utils.FileOperations import makePath, removeDirectory
 
+from .SearchModes import (
+    SearchModeBase,
+    SearchModeByPattern,
+    SearchModeCoverage,
+    SearchModeResume
+)
 
-# Make sure we flush after every print, the "-u" option does more than that
-# and this is easy enough.
-def my_print(*args, **kwargs):
-    print(*args, **kwargs)
-
-    sys.stdout.flush()
-
-# TODO: Use nuitka.utils.Execution instead.
-def check_output(*popenargs, **kwargs):
-    if "stdout" in kwargs:
-        raise ValueError("stdout argument not allowed, it will be overridden.")
-
-    process = subprocess.Popen(
-        stdout = subprocess.PIPE,
-        *popenargs,
-        **kwargs
-    )
-    output, _unused_err = process.communicate()
-    retcode = process.poll()
-
-    if retcode:
-        cmd = kwargs.get("args")
-        if cmd is None:
-            cmd = popenargs[0]
-        raise subprocess.CalledProcessError(retcode, cmd, output = output)
-
-    return output
 
 def check_result(*popenargs, **kwargs):
     if "stdout" in kwargs:
@@ -142,6 +124,8 @@ print(sys.executable);\
         os.environ["COVERAGE_FILE"] = os.path.join(
             os.path.dirname(__file__),
             "..",
+            "..",
+            "..",
             ".coverage"
         )
 
@@ -203,18 +187,28 @@ def convertUsing2to3(path, force = False):
     except shutil.SameFileError:  # @UndefinedVariable
         pass
 
-    # On Windows, we cannot rely on 2to3 to be in the path.
-    if os.name == "nt":
-        command = [
-            sys.executable,
-            os.path.join(
-                os.path.dirname(sys.executable),
-                "Tools/Scripts/2to3.py"
-            )
-        ]
+    # For Python2.6 and 3.2 the -m lib2to3 was not yet supported.
+    use_binary = sys.version_info[:2] in ((2,6), (3,2))
+
+    if use_binary:
+        # On Windows, we cannot rely on 2to3 to be in the path.
+        if os.name == "nt":
+            command = [
+                sys.executable,
+                os.path.join(
+                    os.path.dirname(sys.executable),
+                    "Tools/Scripts/2to3.py"
+                )
+            ]
+        else:
+            command = [
+                "2to3"
+            ]
     else:
         command = [
-            "2to3"
+            sys.executable,
+            "-m",
+            "lib2to3",
         ]
 
     command += [
@@ -230,6 +224,13 @@ def convertUsing2to3(path, force = False):
             stderr = devnull
         )
 
+    with open(new_path) as result_file:
+        data = result_file.read()
+
+    with open(new_path, 'w') as result_file:
+        result_file.write("__file__ = %r\n" % os.path.abspath(path))
+        result_file.write(data)
+
     return new_path, True
 
 
@@ -287,6 +288,24 @@ def decideFilenameVersionSkip(filename):
     return True
 
 
+def _removeCPythonTestSuiteDir():
+    # Cleanup, some tests apparently forget that.
+    try:
+        if os.path.isdir("@test"):
+            removeDirectory("@test", ignore_errors = False)
+        elif os.path.isfile("@test"):
+            os.unlink("@test")
+    except OSError:
+        # TODO: Move this into removeDirectory maybe. Doing an external
+        # call as last resort could be a good idea.
+
+        # This seems to work for broken "lnk" files.
+        if os.name == "nt":
+            os.system("rmdir /S /Q @test")
+
+        if os.path.exists("@test"):
+            raise
+
 def compareWithCPython(dirname, filename, extra_flags, search_mode, needs_2to3):
     """ Call the comparison tool. For a given directory filename.
 
@@ -295,7 +314,6 @@ def compareWithCPython(dirname, filename, extra_flags, search_mode, needs_2to3):
 
     """
 
-    # pylint: disable=too-many-branches
     if dirname is None:
         path = filename
     else:
@@ -319,6 +337,9 @@ def compareWithCPython(dirname, filename, extra_flags, search_mode, needs_2to3):
 
     command += search_mode.getExtraFlags(dirname, filename)
 
+    # Cleanup before and after test stage directory.
+    _removeCPythonTestSuiteDir()
+
     try:
         result = subprocess.call(
             command
@@ -326,22 +347,8 @@ def compareWithCPython(dirname, filename, extra_flags, search_mode, needs_2to3):
     except KeyboardInterrupt:
         result = 2
 
-    # Cleanup, some tests apparently forget that.
-    try:
-        if os.path.isdir("@test"):
-            removeDirectory("@test", ignore_errors = False)
-        elif os.path.isfile("@test"):
-            os.unlink("@test")
-    except OSError:
-        # TODO: Move this into removeDirectory maybe. Doing an external
-        # call as last resort could be a good idea.
-
-        # This seems to work for broken "lnk" files.
-        if os.name == "nt":
-            os.system("rmdir /S /Q @test")
-
-        if os.path.exists("@test"):
-            raise
+    # Cleanup before and after test stage directory.
+    _removeCPythonTestSuiteDir()
 
     if result != 0 and \
        result != 2 and \
@@ -356,6 +363,7 @@ def compareWithCPython(dirname, filename, extra_flags, search_mode, needs_2to3):
         sys.stderr.write("Interrupted, with CTRL-C\n")
         sys.exit(2)
 
+
 def checkCompilesNotWithCPython(dirname, filename, search_mode):
     if dirname is None:
         path = filename
@@ -363,8 +371,8 @@ def checkCompilesNotWithCPython(dirname, filename, search_mode):
         path = os.path.join(dirname, filename)
 
     command = [
-        sys.executable,
-        "-mcompile",
+        _python_executable,
+        "-mcompileall",
         path
     ]
 
@@ -382,6 +390,21 @@ def checkCompilesNotWithCPython(dirname, filename, search_mode):
         sys.exit(result)
 
 
+def checkSucceedsWithCPython(filename):
+    command = [
+        _python_executable,
+        filename
+    ]
+
+    result = subprocess.call(
+        command,
+        stdout = open(os.devnull,'w'),
+        stderr = subprocess.STDOUT
+    )
+
+    return result == 0
+
+
 def hasDebugPython():
     # On Debian systems, these work.
     debug_python = os.path.join("/usr/bin/", os.environ["PYTHON"] + "-dbg")
@@ -414,14 +437,14 @@ def getArchitecture():
         else:
             return "x86"
     else:
-        return os.uname()[4]
+        return os.uname()[4]  # @UndefinedVariable
 
 
 def getDependsExePath():
     if "APPDATA" not in os.environ:
         sys.exit("Error, standalone mode cannot find 'APPDATA' environment.")
 
-    nuitka_app_dir = os.path.join(os.environ["APPDATA"],"nuitka")
+    nuitka_app_dir = getAppDir()
 
     depends_dir = os.path.join(
         nuitka_app_dir,
@@ -612,6 +635,107 @@ Error, needs 'strace' on your system to scan used libraries."""
     return result
 
 
+def checkRuntimeLoadedFilesForOutsideAccesses(loaded_filenames, white_list):
+    # A lot of special white listing is required.
+    # pylint: disable=too-many-branches,too-many-statements
+
+    result = []
+
+    for loaded_filename in loaded_filenames:
+        loaded_filename = os.path.normpath(loaded_filename)
+        loaded_filename = os.path.normcase(loaded_filename)
+        loaded_basename = os.path.basename(loaded_filename)
+
+        ok = False
+        for entry in white_list:
+            if loaded_filename.startswith(entry):
+                ok = True
+
+            while entry:
+                old_entry = entry
+                entry = os.path.dirname(entry)
+
+                if old_entry == entry:
+                    break
+
+                if loaded_filename == entry:
+                    ok = True
+                    break
+        if ok:
+            continue
+
+        if loaded_filename.startswith("/etc/"):
+            continue
+
+        if loaded_filename.startswith("/proc/") or loaded_filename == "/proc":
+            continue
+
+        if loaded_filename.startswith("/dev/"):
+            continue
+
+        if loaded_filename.startswith("/tmp/"):
+            continue
+
+        if loaded_filename.startswith("/run/"):
+            continue
+
+        if loaded_filename.startswith("/sys/"):
+            continue
+
+        if loaded_filename.startswith("/usr/lib/locale/"):
+            continue
+
+        if loaded_filename.startswith("/usr/share/locale/"):
+            continue
+
+        if loaded_filename.startswith("/usr/share/X11/locale/"):
+            continue
+
+        # Themes may of course be loaded.
+        if loaded_filename.startswith("/usr/share/themes"):
+            continue
+        if "gtk" in loaded_filename and "/engines/" in loaded_filename:
+            continue
+
+        # Terminal info files are OK too.
+        if loaded_filename.startswith("/lib/terminfo/"):
+            continue
+
+        # System C libraries are to be expected.
+        if loaded_basename.startswith((
+            "libc.so.",
+            "libpthread.so.",
+            "libdl.so.",
+            "libm.so.",
+        )):
+            continue
+
+        # Taking these from system is harmless and desirable
+        if loaded_basename.startswith((
+            "libz.so",
+            "libutil.so",
+            "libgcc_s.so",
+        )):
+            continue
+
+        # TODO: Unclear, loading gconv from filesystem of installed system
+        # may be OK or not. I think it should be.
+        if loaded_basename == "gconv-modules.cache":
+            continue
+        if "/gconv/" in loaded_filename:
+            continue
+        if loaded_basename.startswith("libicu"):
+            continue
+
+        # GTK may access X files.
+        if loaded_basename == ".Xauthority":
+            continue
+
+        result.append(loaded_filename)
+
+    return result
+
+
 def hasModule(module_name):
     result = subprocess.call(
         (
@@ -713,91 +837,24 @@ def checkReferenceCount(checked_function, max_rounds = 10):
     return result
 
 
+
 def createSearchMode():
     search_mode = len(sys.argv) > 1 and sys.argv[1] == "search"
+    resume_mode = len(sys.argv) > 1 and sys.argv[1] == "resume"
     start_at = sys.argv[2] if len(sys.argv) > 2 else None
     coverage_mode = len(sys.argv) > 1 and sys.argv[1] == "coverage"
 
-    class SearchModeBase(object):
-        def __init__(self):
-            self.may_fail = []
-
-        def consider(self, dirname, filename):
-            # Virtual method, pylint: disable=no-self-use,unused-argument
-            return True
-
-        def finish(self):
-            pass
-
-        def abortOnFinding(self, dirname, filename):
-            for candidate in self.may_fail:
-                if self._match(dirname, filename, candidate):
-                    return False
-
-            return True
-
-        def getExtraFlags(self, dirname, filename):
-            # Virtual method, pylint: disable=no-self-use,unused-argument
-            return []
-
-        def mayFailFor(self, *names):
-            self.may_fail += names
-
-        @classmethod
-        def _match(cls, dirname, filename, candidate):
-            parts = [dirname, filename]
-
-            while None in parts:
-                parts.remove(None)
-            assert parts
-
-            path = os.path.join(*parts)
-
-            return candidate in (
-                dirname,
-                filename,
-                filename.replace(".py", ""),
-                filename.split('.')[0],
-                path,
-                path.replace(".py", ""),
-
-            )
-
-        def isCoverage(self):
-            # Virtual method, pylint: disable=no-self-use
-            return False
 
     if coverage_mode:
-        class SearchModeCoverage(SearchModeBase):
-            def getExtraFlags(self, dirname, filename):
-                return ["coverage"]
-
-            def isCoverage(self):
-                return True
 
         return SearchModeCoverage()
+    elif resume_mode:
+        return SearchModeResume(
+            sys.modules["__main__"].__file__
+        )
     elif search_mode and start_at:
         start_at = start_at.replace('/', os.path.sep)
-
-        class SearchModeByPattern(SearchModeBase):
-            def __init__(self):
-                SearchModeBase.__init__(self)
-
-                self.active = False
-
-            def consider(self, dirname, filename):
-                if self.active:
-                    return True
-
-                self.active = self._match(dirname, filename, start_at)
-                return self.active
-
-            def finish(self):
-                if not self.active:
-                    sys.exit("Error, became never active.")
-
-
-        return SearchModeByPattern()
+        return SearchModeByPattern(start_at)
     else:
         class SearchModeImmediate(SearchModeBase):
             def abortOnFinding(self, dirname, filename):
@@ -858,17 +915,38 @@ def executeReferenceChecked(prefix, names, tests_skipped, tests_stderr):
     gc.enable()
     return result
 
+
 def checkDebugPython():
     if not hasattr(sys, "gettotalrefcount"):
         my_print("Warning, using non-debug Python makes this test ineffective.")
         sys.gettotalrefcount = lambda : 0
 
-@contextmanager
-def withPythonPathChange(python_path):
+
+def addToPythonPath(python_path):
     if type(python_path) in (tuple, list):
         python_path = os.pathsep.join(python_path)
 
     if python_path:
+        if "PYTHONPATH" in os.environ:
+            os.environ["PYTHONPATH"] += os.pathsep + python_path
+        else:
+            os.environ["PYTHONPATH"] = python_path
+
+
+@contextmanager
+def withPythonPathChange(python_path):
+    if python_path:
+        if type(python_path) not in (tuple, list):
+            python_path = python_path.split(os.pathsep)
+
+        python_path = [
+            os.path.normpath(os.path.abspath(element))
+            for element in
+            python_path
+        ]
+
+        python_path = os.pathsep.join(python_path)
+
         if "PYTHONPATH" in os.environ:
             old_path = os.environ["PYTHONPATH"]
             os.environ["PYTHONPATH"] += os.pathsep + python_path
@@ -876,6 +954,13 @@ def withPythonPathChange(python_path):
             old_path = None
             os.environ["PYTHONPATH"] = python_path
 
+#     print(
+#         "Effective PYTHONPATH in %s is %r" % (
+#             sys.modules["__main__"],
+#             os.environ.get("PYTHONPATH", "")
+#         )
+#     )
+
     yield
 
     if python_path:
@@ -1051,6 +1136,9 @@ def compileLibraryPath(search_mode, path, stage_dir, decide, action):
 
 
 def compileLibraryTest(search_mode, stage_dir, decide, action):
+    if not os.path.exists(stage_dir):
+        os.makedirs(stage_dir)
+
     my_dirname = os.path.join(os.path.dirname(__file__), "../../..")
     my_dirname = os.path.normpath(my_dirname)
 
@@ -1065,8 +1153,8 @@ def compileLibraryTest(search_mode, stage_dir, decide, action):
     for path in paths:
         my_print(path)
 
-
     for path in paths:
+        print("Checking path:", path)
         compileLibraryPath(
             search_mode = search_mode,
             path        = path,
@@ -1089,10 +1177,14 @@ def run_async(coro):
             break
     return values, result
 
+
 def async_iterate(g):
     """ Execute async generator until it's done. """
 
-    # Test code, pylint: disable=broad-except,undefined-variable
+    # Test code for Python3, catches all kinds of exceptions.
+    # pylint: disable=broad-except
+
+    # Also Python3 only, pylint: disable=I0021,undefined-variable
 
     res = []
     while True:
@@ -1111,3 +1203,23 @@ def async_iterate(g):
             res.append(str(type(ex)))
 
     return res
+
+
+def getTestingCacheDir():
+    cache_dir = getCacheDir()
+
+    result = os.path.join(cache_dir, "tests_state")
+    makePath(result)
+    return result
+
+
+@contextmanager
+def withDirectoryChange(path, allow_none = False):
+    if path is not None or not allow_none:
+        old_cwd = os.getcwd()
+        os.chdir(path)
+
+    yield
+
+    if path is not None or not allow_none:
+        os.chdir(old_cwd)
