@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -43,7 +43,12 @@ def visitFunction(function, visitor):
     visitor.onLeaveNode(function)
 
 
-class VisitorNoopMixin:
+def visitModule(module, visitor):
+    visitor.onEnterNode(module)
+    visitor.onLeaveNode(module)
+
+
+class VisitorNoopMixin(object):
     def onEnterNode(self, node):
         """ Overloaded for operation before the node children were done. """
         pass
