@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -33,8 +33,12 @@ from nuitka.nodes.OutlineNodes import ExpressionOutlineBody
 from nuitka.nodes.ReturnNodes import StatementReturn
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
 
-from .Helpers import buildNode, getKind, makeStatementsSequenceFromStatement
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+from .TreeHelpers import (
+    buildNode,
+    getKind,
+    makeStatementsSequenceFromStatement
+)
 
 
 def buildComparisonNode(provider, node, source_ref):
