@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -47,7 +47,14 @@ from nuitka.nodes.TryNodes import StatementTry
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
 from nuitka.PythonVersions import python_version
 
-from .Helpers import (
+from .ReformulationAssignmentStatements import (
+    buildAssignmentStatements,
+    buildDeleteStatementFromDecoded,
+    decodeAssignTarget
+)
+from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+from .SyntaxErrors import raiseSyntaxError
+from .TreeHelpers import (
     buildNode,
     buildStatementsNode,
     makeReraiseExceptionStatement,
@@ -56,13 +63,6 @@ from .Helpers import (
     makeStatementsSequenceFromStatements,
     mergeStatements
 )
-from .ReformulationAssignmentStatements import (
-    buildAssignmentStatements,
-    buildDeleteStatementFromDecoded,
-    decodeAssignTarget
-)
-from .ReformulationTryFinallyStatements import makeTryFinallyStatement
-from .SyntaxErrors import raiseSyntaxError
 
 
 def makeTryExceptNoRaise(provider, temp_scope, tried, handling, no_raise,
@@ -328,6 +328,7 @@ def buildTryExceptionNode(provider, node, source_ref):
                         source_ref = source_ref
                     ),
                     final      = buildDeleteStatementFromDecoded(
+                        provider   = provider,
                         kind       = kind,
                         detail     = detail,
                         source_ref = source_ref
