@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -52,7 +52,12 @@ from nuitka.nodes.StatementNodes import (
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
 from nuitka.PythonVersions import python_version
 
-from .Helpers import (
+from .ReformulationAssignmentStatements import buildAssignmentStatements
+from .ReformulationTryExceptStatements import (
+    makeTryExceptSingleHandlerNodeWithPublish
+)
+from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+from .TreeHelpers import (
     buildNode,
     buildStatementsNode,
     getKind,
@@ -61,11 +66,6 @@ from .Helpers import (
     makeStatementsSequence,
     makeStatementsSequenceFromStatement
 )
-from .ReformulationAssignmentStatements import buildAssignmentStatements
-from .ReformulationTryExceptStatements import (
-    makeTryExceptSingleHandlerNodeWithPublish
-)
-from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 
 
 def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
