@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,8 +28,8 @@ from nuitka.nodes.ConstantRefNodes import ExpressionConstantNoneRef
 from nuitka.nodes.YieldNodes import ExpressionYield, ExpressionYieldFrom
 from nuitka.PythonVersions import python_version
 
-from .Helpers import buildNode
 from .SyntaxErrors import raiseSyntaxError
+from .TreeHelpers import buildNode
 
 
 def _checkInsideGenerator(provider, node, source_ref):
