@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -27,6 +27,9 @@ def formatOutput(e):
 
         if message is None and colno is not None:
             colno = None
+
+        if lineno is not None and lineno == 0:
+            lineno = 1
     else:
         reason, = e.args
 
