@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -81,9 +81,13 @@ def listDir(path):
 
 
 def getFileList(path):
+    result = []
+
     for root, _dirnames, filenames in os.walk(path):
         for filename in filenames:
-            yield os.path.join(root, filename)
+            result.append(os.path.join(root, filename))
+
+    return result
 
 
 def getSubDirectories(path):
