@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -82,7 +82,7 @@ def encodeNonAscii(var_name):
     """ Encode variable name that is potentially not ASCII to ASCII only.
 
         For Python3, unicode identifiers can be used, but these are not
-        possible in C++03, so we need to replace them.
+        possible in C, so we need to replace them.
     """
     if python_version < 300:
         return var_name
